@@ -17,9 +17,13 @@
  * along with 100 Boxes. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <locale.h>
+#include <libintl.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <hildon/hildon.h>
 #include <libosso.h>
+#include <gconf/gconf-client.h>
 #include <curl/curl.h>
 #include <string.h>
 #include <libxml/parser.h>
@@ -38,10 +42,10 @@ static void submit_score(struct game_data *d, gchar *name);
 static size_t store_post_reply(void *buffer, size_t size, size_t nmemb, void *userp);
 static void show_high_score(struct game_data *d);
 static void show_error_message(struct game_data *d, const gchar *msg);
+static void save_state(gpointer data);
+static void sound_button_toggled(HildonCheckButton *button, gpointer data);
 
 int main (int argc, char **argv) {
-	osso_context_t *osso_context;
-	HildonProgram *program;
 	GtkWidget *win;
 	GtkWidget *box_table;
 	GtkWidget *box[100];
@@ -49,8 +53,13 @@ int main (int argc, char **argv) {
 	GtkWidget *panel;
 	GtkWidget *new_button;
 	GtkWidget *score_label;
+	GtkWidget *sound_button;
 	GtkWidget *guide_button;
+	GError *error = NULL;
 	gint i,j;
+	GSList *seq;
+	GSList *current;
+	gint len;
 	struct state s;
 	struct game_data d;
 	d.s = &s;
@@ -58,6 +67,12 @@ int main (int argc, char **argv) {
 	gchar file[8];
 	gint err;
 
+	// Initialize i18n
+	setlocale(LC_ALL, "");
+	bindtextdomain(GETTEXT_PACKAGE, localedir);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
+
 	// Initialize sound
 	sound_init(&argc, &argv);
 	d.sounds = add_stock_sound(NULL, "START", SOUND_START, &err);
@@ -67,12 +82,20 @@ int main (int argc, char **argv) {
 	d.sounds = add_stock_sound(d.sounds, "ERROR", SOUND_ERROR, &err);
 
 	// Initialize OSSO and Hildon
-	osso_context = osso_initialize(OSSO_SERVICE, VERSION, TRUE, NULL);
-	if (osso_context == NULL) {
+	d.osso_context = osso_initialize(OSSO_SERVICE, VERSION, TRUE, NULL);
+	if (d.osso_context == NULL) {
         	return OSSO_ERROR;
 	}
 	hildon_gtk_init(&argc, &argv);
-	program = hildon_program_get_instance();
+	osso_application_set_autosave_cb(d.osso_context, save_state, &d);
+	//g_debug("System init: done");
+
+	// Initializie gconf and load application state and preferences
+	gconf_init(argc, argv, NULL);
+	d.client = gconf_client_get_default();
+	s.played = gconf_client_get_int(d.client, KEY_PLAYED_GAMES, &error);
+	s.total = gconf_client_get_int(d.client, KEY_TOTAL_SCORE, &error);
+	s.best = gconf_client_get_int(d.client, KEY_BEST_SCORE, &error);
 
 	// Create the main window
 	win = hildon_window_new();
@@ -92,43 +115,104 @@ int main (int argc, char **argv) {
 
 	// Create other GUI elements
 	container = gtk_hbox_new(FALSE, PADDING);
-	panel = gtk_vbox_new(TRUE, 2*PADDING);
+	panel = gtk_vbox_new(TRUE, PADDING);
 	gtk_widget_set_size_request(GTK_WIDGET(panel), WIDTH-HEIGHT-PADDING, -1);
 
-	new_button = gtk_button_new_with_label("New game");
+	new_button = gtk_button_new_with_label(_("New game"));
 	hildon_helper_set_logical_font(GTK_WIDGET(new_button), "LargeSystemFont");
 	g_signal_connect(GTK_WIDGET(new_button), "clicked", G_CALLBACK(new_clicked), (gpointer)&d);
 	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(new_button));
 
-	score_label = gtk_label_new("Score: 0");
+	score_label = gtk_label_new(_("Score: 0"));
 	hildon_helper_set_logical_font(GTK_WIDGET(score_label), "LargeSystemFont");
 	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(score_label));
 	d.score = score_label;
 
-	guide_button = gtk_button_new_with_label("Guide");
-	//hildon_helper_set_logical_font(GTK_WIDGET(guide_button), "SmallSystemFont");
+	sound_button = hildon_check_button_new(HILDON_SIZE_AUTO);
+	gtk_button_set_label(GTK_BUTTON(sound_button), _("Sound"));
+	s.sound_on = ! gconf_client_get_bool(d.client, KEY_MUTE, NULL);
+	hildon_check_button_set_active(HILDON_CHECK_BUTTON(sound_button), s.sound_on);
+	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(sound_button));
+	g_signal_connect(sound_button, "toggled", G_CALLBACK(sound_button_toggled), (gpointer)&d);
+
+	guide_button = gtk_button_new_with_label(_("Guide"));
 	g_signal_connect(GTK_WIDGET(guide_button), "clicked", G_CALLBACK(guide_clicked), (gpointer)&d);
 	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(guide_button));
+	//g_debug("GUI creation: done");
 
-	new_game(&d);
+	// Initialize game state
+	seq = gconf_client_get_list(d.client, KEY_GAME_SEQ, GCONF_VALUE_INT, NULL);
+	if (seq != NULL) {
+		s.score = 0;
+		s.last = -1;
+		s.error = FALSE;
+		s.active = gconf_client_get_bool(d.client, KEY_ACTIVE, &error);
+		if (error != NULL) {
+			g_error_free(error);
+			error = NULL;
+			new_game(&d);
+		} else {
+			play_stock_sound(d.sounds, "START");
+			current = seq;
+			i = 0;
+			while (i<100 && current != NULL) {
+				s.b[i] = (gint)GINT_TO_POINTER(current->data);
+				set_box_image(&d, i, s.b[i]);
+				if (s.b[i] <= 100 && s.b[i] > s.score) {
+					s.score = s.b[i];
+					s.last = i;
+				}
+			current = g_slist_next(current);
+			i++;
+			}
+			g_slist_free(seq);
+		}
+		//g_debug("Game state restore: done");
+	} else {
+		new_game(&d);
+	}
 
 	// Assemble GUI elements
 	gtk_container_add(GTK_CONTAINER(container), GTK_WIDGET(box_table));
 	gtk_container_add(GTK_CONTAINER(container), GTK_WIDGET(panel));
 	gtk_container_add(GTK_CONTAINER(win), GTK_WIDGET(container));
 	g_signal_connect(win, "destroy", G_CALLBACK(gtk_main_quit), NULL);
-
 	gtk_widget_show_all(win);
+
+	// Main event loop
 	gtk_main();
 
+	osso_application_autosave_force(d.osso_context);
+
 	// Deinitialize OSSO
-	osso_deinitialize(osso_context);
+	osso_deinitialize(d.osso_context);
 	
 	// Deinitialize sound
 	clear_stock(d.sounds);
 	return 0;
 }
 
+static void save_state(gpointer data) {
+	struct game_data* d = (struct game_data *)data;
+	GSList *seq, *last;
+	gint i;
+
+	for (i=0; i<100; i++) {
+		seq = g_slist_prepend( seq, GINT_TO_POINTER(d->s->b[i]) );	
+	}
+	last = g_slist_nth(seq, 99);
+	last->next = NULL;
+	seq = g_slist_reverse(seq);
+	gconf_client_set_list(d->client, KEY_GAME_SEQ, GCONF_VALUE_INT, seq, NULL);
+	g_slist_free(seq);
+	gconf_client_set_bool(d->client, KEY_ACTIVE, d->s->active, NULL);
+	gconf_client_set_bool(d->client, KEY_MUTE, !(d->s->sound_on), NULL);
+	gconf_client_set_int(d->client, KEY_PLAYED_GAMES, d->s->played, NULL);
+	gconf_client_set_int(d->client, KEY_BEST_SCORE, d->s->best, NULL);
+	gconf_client_set_int(d->client, KEY_TOTAL_SCORE, d->s->total, NULL);
+	//g_debug("State saved");
+}
+
 static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data) {
 	gint distx, disty, x, y, poss, n, r, c, old;
 	gchar label[4], score_label[11];
@@ -141,15 +225,15 @@ static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data)
 	}
 	r = n / 10;
 	c = n % 10;
-	if (d->s->active) {
+	if (d->s->active && !(d->s->error)) {
 		d->s->last = n;
 		if ( d->s->b[n] == CLICKABLE) {
-			play_stock_sound(d->sounds, "CLICK");
+			if (d->s->sound_on) play_stock_sound(d->sounds, "CLICK");
 			d->s->score++;
 			d->s->b[n] = d->s->score;
 			sprintf(label, "%d", d->s->score);
 			set_box_image(d, n, d->s->score);
-			sprintf(score_label, "Score: %d", d->s->score);
+			sprintf(score_label, _("Score: %d"), d->s->score);
 			gtk_label_set_text(GTK_LABEL(d->score), score_label);
 			poss = 0;
 			for (x=0; x<10; x++) {
@@ -178,16 +262,22 @@ static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data)
 			if (poss == 0) {
 				d->s->active = FALSE;
 				if (d->s->score < 100) {
-					play_stock_sound(d->sounds, "LOSE");
+					if (d->s->sound_on) play_stock_sound(d->sounds, "LOSE");
 				} else {
-					play_stock_sound(d->sounds, "WIN");
+					if (d->s->sound_on) play_stock_sound(d->sounds, "WIN");
+				}
+				d->s->played++;
+				d->s->total += d->s->score;
+				if (d->s->score > d->s->best) {
+					d->s->best = d->s->score;
 				}
 				g_timeout_add_seconds(2, show_end_dialog, (gpointer)d);
 			}
+			osso_application_userdata_changed(d->osso_context);
 		} else {
-			play_stock_sound(d->sounds, "ERROR");
+			if (d->s->sound_on) play_stock_sound(d->sounds, "ERROR");
 			set_box_image(d, n, ERROR);
-			d->s->active = FALSE;
+			d->s->error = TRUE;
 			g_timeout_add_seconds(1, restore_box_image, (gpointer)d);
 		}
 	}
@@ -196,24 +286,33 @@ static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data)
 
 static void new_clicked(GtkWidget *b, gpointer data) {
 	struct game_data *d = (struct game_data *)data;
+	osso_application_userdata_changed(d->osso_context);
 	new_game(d);
 }
 
+static void sound_button_toggled (HildonCheckButton *button, gpointer data) {
+	struct game_data *d = (struct game_data *)data;
+	d->s->sound_on = hildon_check_button_get_active(button);
+	osso_application_userdata_changed(d->osso_context);
+}
+
 static void guide_clicked(GtkWidget *b, gpointer data) {
-	system("dbus-send --print-reply --dest=com.nokia.osso_browser /com/nokia/osso_browser/service com.nokia.osso_browser.open_new_window string:file:///usr/share/100boxes/index.html");
+	system(_("dbus-send --print-reply --dest=com.nokia.osso_browser /com/nokia/osso_browser/service com.nokia.osso_browser.open_new_window string:file:///usr/share/100boxes/index.html"));
 }
 
 void new_game(struct game_data *d) {
 	gint i;
 	d->s->active = TRUE;
+	d->s->error = FALSE;
 	d->s->score = 0;
 	d->s->last = -1;
-	play_stock_sound(d->sounds, "START");
+	//g_debug("Starting new game");
+	if (d->s->sound_on) play_stock_sound(d->sounds, "START");
 	for (i=0; i<100; i++) {
 		d->s->b[i] = CLICKABLE;
 		set_box_image(d, i, CLICKABLE);
 	}
-	gtk_label_set_text(GTK_LABEL(d->score), "Score: 0");
+	gtk_label_set_text(GTK_LABEL(d->score), _("Score: 0"));
 }
 
 static void set_box_image(struct game_data *d, gint box_index, gint img_index) {
@@ -235,10 +334,14 @@ static gboolean show_end_dialog(gpointer data) {
 	GtkWidget *comment;
 	GtkWidget *icon;
 	GtkWidget *caption;
+	GtkWidget *played_games;
+	GtkWidget *best_score;
+	GtkWidget *avg_score;
 	GtkWidget *submit;
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *dialog;
+	gchar line[80];
 	gchar file[40];
 	gchar *message;
 	struct game_data *d = (struct game_data *)data;
@@ -252,11 +355,12 @@ static gboolean show_end_dialog(gpointer data) {
                                                   NULL);
 
 	if (d->s->score < 100) {
-		gtk_window_set_title(GTK_WINDOW(dialog), "You lose!");
+		gtk_window_set_title(GTK_WINDOW(dialog), _("You lose!"));
 	} else {
-		gtk_window_set_title(GTK_WINDOW(dialog), "You win!");
+		gtk_window_set_title(GTK_WINDOW(dialog), _("You win!"));
 	}
 	area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+	gtk_box_set_spacing(GTK_BOX(area), DIALOG_PADDING);
 
 	comment = gtk_hbox_new(FALSE, 5);
 	sprintf(file, "/usr/share/pixmaps/100boxes/%d.png", d->s->score);
@@ -269,8 +373,24 @@ static gboolean show_end_dialog(gpointer data) {
 	gtk_container_add(GTK_CONTAINER(comment), GTK_WIDGET(caption));
 	gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(comment));		
 
+	best_score = gtk_label_new("");
+	sprintf(line, _("Your best score: <b>%d</b>"), d->s->best);
+	gtk_misc_set_alignment(GTK_MISC(best_score), 0.0, 0.5);
+	gtk_label_set_markup(GTK_LABEL(best_score), line);
+	gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(best_score));
+	played_games = gtk_label_new("");
+	sprintf(line, _("Games played: <b>%d</b>"), d->s->played);
+	gtk_misc_set_alignment(GTK_MISC(played_games), 0.0, 0.5);
+	gtk_label_set_markup(GTK_LABEL(played_games), line);
+	gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(played_games));
+	avg_score = gtk_label_new("");
+	sprintf(line, _("Average score: <b>%d</b>"), (d->s->total / d->s->played));
+	gtk_misc_set_alignment(GTK_MISC(avg_score), 0.0, 0.5);
+	gtk_label_set_markup(GTK_LABEL(avg_score), line);
+	gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(avg_score));
+
 	submit = gtk_hbox_new(FALSE, 5);
-	label = gtk_label_new("Enter your name to submit your score to worldwide high score table:");
+	label = gtk_label_new(_("Enter your name to submit your score to worldwide high score table:"));
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	gtk_container_add(GTK_CONTAINER(submit), GTK_WIDGET(label));
 	entry = gtk_entry_new();
@@ -305,13 +425,13 @@ static gchar* create_end_message(gint score) {
 	gint l;
 	gint n = 7;
 	gint s[] = {40, 60, 75, 90, 97, 99, 100};
-	gchar* m[] = { 	"You can do better than this!", 
-			"Train some more to improve.",
-			"Nice try, the basics are there.",
-			"You are approaching the solution.",
-			"Good, you are near the goal!",
-			"Oh, you were so close!",
-			"You are a 100 Boxes champion!" };
+	gchar* m[] = { 	_("You can do better than this!"), 
+			_("Train some more to improve."),
+			_("Nice try, the basics are there."),
+			_("You are approaching the solution."),
+			_("Good, you are near the goal!"),
+			_("Oh, you were so close!"),
+			_("You are a 100 Boxes champion!") };
 	gchar *msg;
 	for (i=0; i<n; i++) {
 		if (score <= s[i]) {
@@ -329,7 +449,7 @@ static gboolean restore_box_image(gpointer data) {
 	if (d->s->score > 0) {	// Cannot restore if game not started
 		gint n = d->s->last;
 		set_box_image(d, n, d->s->b[n]);
-		d->s->active = TRUE;
+		d->s->error = FALSE;
 	}
 	return FALSE; // no repeat		
 }
@@ -356,11 +476,11 @@ static void submit_score(struct game_data *d, gchar *name) {
 			//g_debug("REPLY:\n%s", d->post_reply);
 			show_high_score(d);
 		} else {
-			show_error_message(d, "Submission failed.");
+			show_error_message(d, _("Submission failed."));
 		}
 		curl_easy_cleanup(d->curl);
 	} else {
-		show_error_message(d, "Connection failed.");
+		show_error_message(d, _("Connection failed."));
 	}
 }
 
@@ -368,8 +488,6 @@ static size_t store_post_reply(void *buffer, size_t size, size_t nmemb, void *us
 	struct game_data *d = (struct game_data *)userp;
 	gchar *part = (gchar *)buffer;
 	size_t len = size * nmemb;
-	//g_debug("In store_post_reply. Size=%d:", len);
-	//g_debug("In store_post_reply. post_reply=%s:", d->post_reply);
 	if (d->post_reply == NULL) {
 		d->post_reply = (gchar*) g_malloc(len + 1);
 		strcpy(d->post_reply, part);
@@ -377,7 +495,6 @@ static size_t store_post_reply(void *buffer, size_t size, size_t nmemb, void *us
 		d->post_reply = (gchar*) g_realloc(d->post_reply, strlen(d->post_reply) + len);
 		strcat(d->post_reply, part);
 	}
-	//g_debug("Exiting store_post_reply. post_reply=%s:", d->post_reply);
 	return len;
 }
 
@@ -390,27 +507,56 @@ static void show_error_message(struct game_data *d, const gchar *msg) {
 static void show_high_score(struct game_data *d) {
 	xmlDoc *doc;
 	xmlNode *root, *pos, *tot, *top, *rec;
+	xmlNode *week, *w_pos, *w_tot, *w_top, *w_rec;
+	xmlNode *day, *d_pos, *d_tot, *d_top, *d_rec;
 	gchar *line, *pos_v, *tot_v;
-	gchar *n_v[10], *name_v[10], *score_v[10], *date_v[10];
+	gchar *n_v[MAX_HIGH_SCORES], *name_v[MAX_HIGH_SCORES], *score_v[MAX_HIGH_SCORES], *date_v[MAX_HIGH_SCORES];
+	gchar *w_line, *w_pos_v, *w_tot_v;
+	gchar *w_n_v[MAX_HIGH_SCORES], *w_name_v[MAX_HIGH_SCORES], *w_score_v[MAX_HIGH_SCORES], *w_date_v[MAX_HIGH_SCORES];
+	gchar *d_line, *d_pos_v, *d_tot_v;
+	gchar *d_n_v[MAX_HIGH_SCORES], *d_name_v[MAX_HIGH_SCORES], *d_score_v[MAX_HIGH_SCORES], *d_date_v[MAX_HIGH_SCORES];
 	gint i, j;
+	GtkWidget *tabs;
+	GtkWidget *day_area;
+	GtkWidget *week_area;
+	GtkWidget *all_area;
 	GtkWidget *text_l, *n_h, *name_h, *score_h, *date_h;
-	GtkWidget *n_l[10], *name_l[10], *score_l[10], *date_l[10];
-	GtkWidget *area;
+	GtkWidget *n_l[MAX_HIGH_SCORES], *name_l[MAX_HIGH_SCORES], *score_l[MAX_HIGH_SCORES], *date_l[MAX_HIGH_SCORES];
+	GtkWidget *label;
 	GtkWidget *table;
+	GtkWidget *w_text_l, *w_n_h, *w_name_h, *w_score_h, *w_date_h;
+	GtkWidget *w_n_l[MAX_HIGH_SCORES], *w_name_l[MAX_HIGH_SCORES], *w_score_l[MAX_HIGH_SCORES], *w_date_l[MAX_HIGH_SCORES];
+	GtkWidget *w_label;
+	GtkWidget *w_table;
+	GtkWidget *d_text_l, *d_n_h, *d_name_h, *d_score_h;
+	GtkWidget *d_n_l[MAX_HIGH_SCORES], *d_name_l[MAX_HIGH_SCORES], *d_score_l[MAX_HIGH_SCORES];
+	GtkWidget *d_label;
+	GtkWidget *d_table;
+	GtkWidget *area;
 	GtkWidget *dialog;
 
 	doc = xmlReadDoc((xmlChar*)d->post_reply, NULL, NULL, 0);
 	if (doc == NULL) {
-		show_error_message(d, "Bad response received");
+		show_error_message(d, _("Bad response received."));
 	} else {
-		dialog = gtk_dialog_new();
-		dialog = gtk_dialog_new_with_buttons ("High scores", GTK_WINDOW(d->window),
+		dialog = gtk_dialog_new_with_buttons (_("High scores"), GTK_WINDOW(d->window),
                                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                                   GTK_STOCK_OK,
                                                   GTK_RESPONSE_CLOSE,
                                                   NULL);
 
 		area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+		tabs = gtk_notebook_new();
+ 		gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(tabs));
+		d_label = gtk_label_new( _("Today"));
+		day_area = gtk_vbox_new(FALSE, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(tabs), GTK_WIDGET(day_area), d_label);
+		w_label = gtk_label_new( _("7 Days"));
+		week_area = gtk_vbox_new(FALSE, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(tabs), GTK_WIDGET(week_area), w_label);
+		label = gtk_label_new( _("All time"));
+		all_area = gtk_vbox_new(FALSE, 0);
+		gtk_notebook_append_page(GTK_NOTEBOOK(tabs), GTK_WIDGET(all_area), label);
 
 		//g_debug("Getting root XML element");
 		root = xmlDocGetRootElement(doc);
@@ -421,14 +567,13 @@ static void show_high_score(struct game_data *d) {
 		top = tot->next;
 
 		line = (gchar*) g_malloc(100);
-		sprintf(line, "<big>You got place no. <b>%s</b> of <b>%s</b>!</big>", pos_v, tot_v);
-
+		sprintf(line, _("<big>You got place no. <b>%s</b> of <b>%s</b>!</big>"), pos_v, tot_v);
 		text_l = gtk_label_new("");
- 		gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(text_l));
+ 		gtk_container_add(GTK_CONTAINER(all_area), GTK_WIDGET(text_l));
 		gtk_label_set_markup(GTK_LABEL(text_l), line);
 
 		// Extract top scores
-		for (rec=top->children, i=0; rec!=NULL && i<10; rec = rec->next, i++) {
+		for (rec=top->children, i=0; rec!=NULL && i<MAX_HIGH_SCORES; rec = rec->next, i++) {
 			n_v[i] = (gchar*) g_malloc(3);
 			sprintf(n_v[i], "%d", (i+1));
 			name_v[i] = xmlGetProp(rec, "name");
@@ -438,19 +583,19 @@ static void show_high_score(struct game_data *d) {
 
 		// Create table and header row
 		table = gtk_table_new(4, i+1, FALSE);
- 		gtk_container_add(GTK_CONTAINER(area), GTK_WIDGET(table));
+ 		gtk_container_add(GTK_CONTAINER(all_area), GTK_WIDGET(table));
 		n_h = gtk_label_new("");
  		gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(n_h), 0, 1, 0, 1);
-		gtk_label_set_markup(GTK_LABEL(n_h), "<b>Place</b>");
+		gtk_label_set_markup(GTK_LABEL(n_h), _("<b>Place</b>"));
 		name_h = gtk_label_new("");
  		gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(name_h), 1, 2, 0, 1);
-		gtk_label_set_markup(GTK_LABEL(name_h), "<b>Name</b>");
+		gtk_label_set_markup(GTK_LABEL(name_h), _("<b>Name</b>"));
 		score_h = gtk_label_new("");
  		gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(score_h), 2, 3, 0, 1);
-		gtk_label_set_markup(GTK_LABEL(score_h), "<b>Score</b>");
+		gtk_label_set_markup(GTK_LABEL(score_h), _("<b>Score</b>"));
 		date_h = gtk_label_new("");
  		gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(date_h), 3, 4, 0, 1);
-		gtk_label_set_markup(GTK_LABEL(date_h), "<b>Date</b>");
+		gtk_label_set_markup(GTK_LABEL(date_h), _("<b>Date</b>"));
 
 		// Create data rows
 		for (j=0; j<i; j++) {
@@ -464,6 +609,102 @@ static void show_high_score(struct game_data *d) {
 			gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(date_l[j]), 3, 4, j+1, j+2);
 		}
 
+		//g_debug("Getting week XML element");
+		week = top->next;
+		w_pos = week->children;
+		w_pos_v = (gchar*) w_pos->children->content;
+		w_tot = w_pos->next;
+		w_tot_v = (gchar*) w_tot->children->content;
+		w_top = w_tot->next;
+
+		w_line = (gchar*) g_malloc(100);
+		sprintf(w_line, _("<big>You got place no. <b>%s</b> of <b>%s</b>!</big>"), w_pos_v, w_tot_v);
+		w_text_l = gtk_label_new("");
+ 		gtk_container_add(GTK_CONTAINER(week_area), GTK_WIDGET(w_text_l));
+		gtk_label_set_markup(GTK_LABEL(w_text_l), w_line);
+
+		// Extract top scores
+		for (w_rec=w_top->children, i=0; w_rec!=NULL && i<MAX_HIGH_SCORES; w_rec = w_rec->next, i++) {
+			w_n_v[i] = (gchar*) g_malloc(3);
+			sprintf(w_n_v[i], "%d", (i+1));
+			w_name_v[i] = xmlGetProp(w_rec, "name");
+			w_score_v[i] = xmlGetProp(w_rec, "score");
+			w_date_v[i] = xmlGetProp(w_rec, "date");
+		}
+
+		// Create table and header row
+		w_table = gtk_table_new(4, i+1, FALSE);
+ 		gtk_container_add(GTK_CONTAINER(week_area), GTK_WIDGET(w_table));
+		w_n_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_n_h), 0, 1, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(w_n_h), _("<b>Place</b>"));
+		w_name_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_name_h), 1, 2, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(w_name_h), _("<b>Name</b>"));
+		w_score_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_score_h), 2, 3, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(w_score_h), _("<b>Score</b>"));
+		w_date_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_date_h), 3, 4, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(w_date_h), _("<b>Date</b>"));
+
+		// Create data rows
+		for (j=0; j<i; j++) {
+			w_n_l[j] = gtk_label_new(w_n_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_n_l[j]), 0, 1, j+1, j+2);
+			w_name_l[j] = gtk_label_new(w_name_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_name_l[j]), 1, 2, j+1, j+2);
+			w_score_l[j] = gtk_label_new(w_score_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_score_l[j]), 2, 3, j+1, j+2);
+			w_date_l[j] = gtk_label_new(w_date_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_date_l[j]), 3, 4, j+1, j+2);
+		}
+
+		//g_debug("Getting day XML element");
+		day = week->next;
+		d_pos = day->children;
+		d_pos_v = (gchar*) d_pos->children->content;
+		d_tot = d_pos->next;
+		d_tot_v = (gchar*) d_tot->children->content;
+		d_top = d_tot->next;
+
+		d_line = (gchar*) g_malloc(100);
+		sprintf(d_line, _("<big>You got place no. <b>%s</b> of <b>%s</b>!</big>"), d_pos_v, d_tot_v);
+		d_text_l = gtk_label_new("");
+ 		gtk_container_add(GTK_CONTAINER(day_area), GTK_WIDGET(d_text_l));
+		gtk_label_set_markup(GTK_LABEL(d_text_l), d_line);
+
+		// Extract top scores
+		for (d_rec=d_top->children, i=0; d_rec!=NULL && i<MAX_HIGH_SCORES; d_rec = d_rec->next, i++) {
+			d_n_v[i] = (gchar*) g_malloc(3);
+			sprintf(d_n_v[i], "%d", (i+1));
+			d_name_v[i] = xmlGetProp(d_rec, "name");
+			d_score_v[i] = xmlGetProp(d_rec, "score");
+		}
+
+		// Create table and header row
+		d_table = gtk_table_new(3, i+1, FALSE);
+ 		gtk_container_add(GTK_CONTAINER(day_area), GTK_WIDGET(d_table));
+		d_n_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_n_h), 0, 1, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(d_n_h), _("<b>Place</b>"));
+		d_name_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_name_h), 1, 2, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(d_name_h), _("<b>Name</b>"));
+		d_score_h = gtk_label_new("");
+ 		gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_score_h), 2, 3, 0, 1);
+		gtk_label_set_markup(GTK_LABEL(d_score_h), _("<b>Score</b>"));
+
+		// Create data rows
+		for (j=0; j<i; j++) {
+			d_n_l[j] = gtk_label_new(d_n_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_n_l[j]), 0, 1, j+1, j+2);
+			d_name_l[j] = gtk_label_new(d_name_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_name_l[j]), 1, 2, j+1, j+2);
+			d_score_l[j] = gtk_label_new(d_score_v[j]);
+			gtk_table_attach_defaults(GTK_TABLE(d_table), GTK_WIDGET(d_score_l[j]), 2, 3, j+1, j+2);
+		}
+
 		gtk_widget_show_all(GTK_WIDGET(dialog));
 		gtk_dialog_run(GTK_DIALOG(dialog));
 
@@ -479,3 +720,4 @@ static void show_high_score(struct game_data *d) {
 	g_free(d->post_reply);
 	d->post_reply = NULL;
 }
+
