@@ -21,6 +21,7 @@
 #define HEIGHT 480
 #define WIDTH 800
 #define PADDING 35
+#define DIALOG_PADDING 15
 #define IMAGES 103
 #define UNUSABLE 0
 #define CLICKABLE 101
@@ -33,21 +34,36 @@
 #define SOUND_WIN "/usr/share/sounds/100boxes/win.mp3"
 #define SOUND_LOSE "/usr/share/sounds/100boxes/lose.mp3"
 #define SOUND_ERROR "/usr/share/sounds/100boxes/error.mp3"
+#define KEY_DIR "/apps/100boxes"
+#define KEY_ACTIVE "/apps/100boxes/active"
+#define KEY_GAME_SEQ "/apps/100boxes/game_sequence"
+#define KEY_MUTE "/apps/100boxes/mute"
+#define KEY_PLAYED_GAMES "/apps/100boxes/played_games"
+#define KEY_BEST_SCORE "/apps/100boxes/best_score"
+#define KEY_TOTAL_SCORE "/apps/100boxes/total_score"
+#define MAX_HIGH_SCORES 8
 
 struct state {
 	gboolean active;
+	gboolean error;
+	gboolean sound_on;
 	gint score;
 	gint b[100];
 	gint last;
+	gint played;
+	gint best;
+	gint total;
 };
 
 struct game_data {
+	osso_context_t *osso_context;
 	struct state *s;
 	GtkWindow *window;
 	GtkWidget **base;
 	GtkWidget *score;
 	CURL *curl;
 	gchar *post_reply;
+	GConfClient *client;
 	GList *sounds;
 };
 
