@@ -46,16 +46,16 @@ static void error_message_received(GstBus *bus, GstMessage *message, GstElement
 static void eos_message_received(GstBus *bus, GstMessage *message, GstElement *pipeline) {
 	/* stop playback and free pipeline */
 	gst_element_set_state(pipeline, GST_STATE_NULL);
-	g_debug("PLAYING FINISHED");
+	//g_debug("PLAYING FINISHED");
 	free_pipeline(pipeline);
 }
 
 static void stock_eos_message_received(GstBus *bus, GstMessage *message, GstElement *pipeline) {
 	/* pause playback and rewind (seek to beginning) */
-	g_debug("PLAYING FINISHED");
+	//g_debug("PLAYING FINISHED");
 	gst_element_set_state(pipeline, GST_STATE_PAUSED);
 	gst_element_seek_simple(pipeline, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT, 0);
-	g_debug("STREAM SOUGHT BACK TO BEGINNING");
+	//g_debug("STREAM SOUGHT BACK TO BEGINNING");
 }
 
 static void free_pipeline(GstElement *pipeline) {
@@ -75,8 +75,8 @@ static struct ss* find_stock_element(GList *stock, const gchar *name) {
 	GList *elem;
 
 	if (stock != NULL) {
-		template = g_malloc(sizeof(struct ss));
-		template->name = name;
+		template = (struct ss *)g_malloc(sizeof(struct ss));
+		template->name = (gchar*)name;
 		elem = g_list_find_custom(stock, template, stock_sound_cmp);
 		g_free(template);
 		if (elem == NULL) {
@@ -122,7 +122,7 @@ int play_sound(const gchar *path) {
 	
 	/* start playback */
 	gst_element_set_state(pipeline, GST_STATE_PLAYING);
-	g_debug("PLAYING STARTED");
+	//g_debug("PLAYING STARTED");
 	return ERR_NONE;	
 }
 
